part of 'url_bloc.dart';

@immutable
sealed class UrlEvent {}

class UrlResetEvent extends UrlEvent {}

class ChooseWayToLoadVideoEvent extends UrlEvent {
  final int selectedIndex;
  ChooseWayToLoadVideoEvent(this.selectedIndex);
}

class ChooseExampleVideoEvent extends UrlEvent {
  final int selectedIndex;
  ChooseExampleVideoEvent(this.selectedIndex);
}

class ChooseWayToGetLinkEvent extends UrlEvent {
  final int selectedIndex;
  ChooseWayToGetLinkEvent(this.selectedIndex);
}

class TypeOwnLinkEvent extends UrlEvent {
  final String urlPath;
  TypeOwnLinkEvent(this.urlPath);
}

class ChooseExtraOptionsEvent extends UrlEvent {
  final int selectedIndex;
  ChooseExtraOptionsEvent(this.selectedIndex);
}

class ChooseVideoFormatEvent extends UrlEvent {
  final int selectedIndex;
  ChooseVideoFormatEvent(this.selectedIndex);
}

class TypeOwnLinkWithoutFormatEvent extends UrlEvent {
  final String urlPath;
  TypeOwnLinkWithoutFormatEvent(this.urlPath);
}

class ChooseConfigFileEvent extends UrlEvent {
  final int selectedIndex;
  ChooseConfigFileEvent(this.selectedIndex);
}

class UpdateOwnLinkWithoutFormatEvent extends UrlEvent {
  final String? urlPath;
  final int? index;
  UpdateOwnLinkWithoutFormatEvent(this.urlPath, this.index);
}